<?php

namespace Bydedi\Geoip;

use GeoIp2\Database\Reader;

class GeoIp
{

    static protected $instance = null;

    /**
     * @var GeoIp2\Database\Reader
     */
    protected $readerCountry = null;
    /**
     * @var GeoIp2\Database\Reader
     */
    protected $readerCity = null;
    /**
     * @var string
     */
    protected $addressIp;

    protected $record;

    /**
     * Contructor
     */
    protected function __construct()
    {
        $this->addressIp = $_SERVER['REMOTE_ADDR'];
    }

    /**
     * @return GeoIp
     */
    public static function getInstance() {
        if (!(self::$instance instanceof self)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Crée le reader pour lire les pays
     */
    protected function createReaderCountry()
    {
        try {
            $this->readerCountry = new Reader(__DIR__ . '/../../data/GeoLite2-Country.mmdb');
        } catch (Exception $e) {
            error_log($e->getMessage());
        }
    }

    /**
     * Crée le reader pour lire les villes
     */
    protected function createReaderCity()
    {
        try {
            $this->readerCity = new Reader(__DIR__ . '/../../data/GeoLite2-City.mmdb');
        } catch (Exception $e) {
            error_log($e->getMessage());
        }
    }

    /**
     * @return bool|GeoIp2\Record\City
     */
    public function getCity()
    {
        if (is_null($this->readerCity)) {
            $this->createReaderCity();
        }

        if (empty($this->record) || empty($this->record->city)) {
            try {
                $this->record = $this->readerCity->city($this->addressIp);
            } catch (Exception $e) {
                error_log($e->getMessage());
                return false;
            }
        }

        return $this->record->city;
    }

    /**
     * @return bool|GeoIp2\Record\Country
     */
    public function getCountry()
    {
        if (is_null($this->readerCountry)) {
            $this->createReaderCountry();
        }

        if (empty($this->record)) {
            try {
                $this->record = $this->readerCountry->country($this->addressIp);
            } catch (Exception $e) {
                error_log($e->getMessage());
                return false;
            }
        }

        return $this->record->country;
    }

    /**
     * @return bool|GeoIp2\Record\Continent
     */
    public function getContinent()
    {
        if (is_null($this->readerCountry)) {
            $this->createReaderCountry();
        }

        if (empty($this->record)) {
            try {
                $this->record = $this->readerCountry->country($this->addressIp);
            } catch (Exception $e) {
                error_log($e->getMessage());
                return false;
            }
        }

        return $this->record->continent;
    }
}
